import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:wordguessing/models/word.dart';
import 'package:wordguessing/utils/random_pick_words.dart';

import '../provider/data.dart';

class GamePickImagePage extends StatelessWidget {
  final int _countWords = 4;

  Future<void> startGame(Data myProvider, String lang) async {
    await resetGame(myProvider);
    myProvider.updateLang = lang;
    await nextWord(myProvider);
  }

  Future<void> resetGame(Data myProvider) async {
    myProvider.updateLang = '';
    myProvider.updateQuestionsCount = 0;
    myProvider.updateGoodAnswers = 0;
    myProvider.updateWrongAnswers = 0;
    myProvider.updateWord = null;
    myProvider.updateImages = [];
  }

  Future<void> nextWord(Data myProvider) async {
    await pickData(myProvider);
    myProvider.updateQuestionsCount = myProvider.questionsCount + 1;
  }

  Future<void> pickData(Data myProvider) async {
    List<Word> words = [];
    RandomPickWords randomPickWords;
    Word? word;

    int attempts = 0;
    do {
      randomPickWords = RandomPickWords();
      await randomPickWords.init(myProvider.lang, _countWords);

      words = randomPickWords.words;

      if (words.length > 0) {
        word = words.take(1).toList()[0];

        if ((words.length == _countWords) && !myProvider.isRecentlyPicked(word.key)) {
          myProvider.updateWord = word;

          List<Word> images = [];
          words.forEach((element) {
            images.add(element);
          });

          myProvider.updateImages = images;
        }
      }

      attempts++;
    } while (myProvider.word != word && attempts < 10);
  }

  Future<void> checkWord(Data myProvider, word) async {
    if (myProvider.word?.key == word.key) {
      myProvider.updateGoodAnswers = myProvider.goodAnswers + 1;
      nextWord(myProvider);
    } else {
      myProvider.updateWrongAnswers = myProvider.wrongAnswers + 1;
    }
  }

  Container _buildScoreItemContainer(String text, Color blockColor) {
    Color backgroundColor = blockColor;

    // Darken block color to get border color
    double amount = 0.2;
    final hsl = HSLColor.fromColor(blockColor);
    final hslDark = hsl.withLightness((hsl.lightness - amount).clamp(0.0, 1.0));
    Color borderColor = hslDark.toColor();

    return Container(
      margin: EdgeInsets.symmetric(horizontal: 15),
      padding: EdgeInsets.all(5),
      decoration: BoxDecoration(
        color: backgroundColor,
        borderRadius: BorderRadius.circular(4),
        border: Border.all(
          color: borderColor,
          width: 4,
        ),
      ),
      child: Text(
        text,
        style: TextStyle(
          fontSize: 25,
          fontWeight: FontWeight.w600,
          color: Colors.black,
        ),
      ),
    );
  }

  String getGoodAnswersString(Data myProvider) {
    int count = myProvider.questionsCount - 1;
    return '👍 ' + myProvider.goodAnswers.toString() + '/' + count.toString();
  }

  String getWrongAnswersString(Data myProvider) {
    int count = myProvider.questionsCount - 1;
    return '🚩 ' + myProvider.wrongAnswers.toString() + '/' + count.toString();
  }

  Container _buildScoreContainer(Data myProvider) {
    return Container(
      padding: EdgeInsets.all(5),
      child: Row(
        mainAxisSize: MainAxisSize.min,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          _buildScoreItemContainer(
            getGoodAnswersString(myProvider),
            Colors.green,
          ),
          SizedBox(width: 20),
          _buildScoreItemContainer(
            getWrongAnswersString(myProvider),
            Colors.orange,
          ),
        ],
      ),
    );
  }

  Container _buildImageContainer(Data myProvider, Word word) {
    double imageSize = 130;

    String imageAsset = 'assets/placeholder.png';
    if ((word.images.length != 0)) {
      imageAsset = 'assets/images/' + word.images[0];
    }

    return Container(
      child: TextButton(
        child: Container(
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(8),
            border: Border.all(
              color: Colors.blue.shade200,
              width: 8,
            ),
          ),
          margin: EdgeInsets.all(2),
          child: Image(
            image: AssetImage(imageAsset),
            width: imageSize,
            height: imageSize,
            fit: BoxFit.fill,
          ),
        ),
        onPressed: () {
          checkWord(myProvider, word);
        },
      ),
    );
  }

  Column _buildImageItemsBlock(Data myProvider) {
    List<Word> images = myProvider.images;

    if (images.length != _countWords) {
      return Column();
    }

    images.sort((a, b) => a.key.compareTo(b.key));

    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        Row(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            _buildImageContainer(myProvider, images[0]),
            _buildImageContainer(myProvider, images[1]),
          ],
        ),
        Row(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            _buildImageContainer(myProvider, images[2]),
            _buildImageContainer(myProvider, images[3]),
          ],
        )
      ],
    );
  }

  Widget _buildTextItemBlock(Data myProvider) {
    Word? word = myProvider.word;

    if (word == null) {
      return const SizedBox.shrink();
    }

    return Container(
      decoration: BoxDecoration(
        color: Colors.blue.shade800,
        borderRadius: BorderRadius.circular(6),
        border: Border.all(
          color: Colors.white,
          width: 6,
        ),
      ),
      child: Padding(
        padding: EdgeInsets.all(12),
        child: Text(
          word.text,
          style: TextStyle(
            fontSize: 30,
            fontWeight: FontWeight.w600,
            color: Colors.white,
          ),
        ),
      ),
    );
  }

  Column _buildStartGameBlock(Data myProvider) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        Container(
          child: ElevatedButton(
            style: ElevatedButton.styleFrom(
              foregroundColor: Colors.teal,
              backgroundColor: Colors.teal,
              padding: EdgeInsets.all(35),
            ),
            child: Text(
              "🇫🇷",
              style: TextStyle(
                fontSize: 60,
                color: Colors.black,
              ),
            ),
            onPressed: () => startGame(myProvider, 'fr'),
          ),
        ),
        SizedBox(height: 15),
        Container(
          child: ElevatedButton(
            style: ElevatedButton.styleFrom(
              foregroundColor: Colors.teal,
              backgroundColor: Colors.teal,
              padding: EdgeInsets.all(35),
            ),
            child: Text(
              "🇬🇧",
              style: TextStyle(
                fontSize: 60,
                color: Colors.black,
              ),
            ),
            onPressed: () => startGame(myProvider, 'en'),
          ),
        )
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    Data myProvider = Provider.of<Data>(context);

    Scaffold pageContent = Scaffold(
      appBar: AppBar(
        elevation: 0,
        actions: <Widget>[
          IconButton(
            icon: const Icon(Icons.loop),
            onPressed: () => resetGame(myProvider),
          ),
        ],
      ),
      backgroundColor: Colors.blue,
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
          crossAxisAlignment: CrossAxisAlignment.center,
          mainAxisSize: MainAxisSize.max,
          children: <Widget>[
            _buildTextItemBlock(myProvider),
            SizedBox(height: 2),
            ((myProvider.word == null) || (myProvider.word?.key == ''))
                ? _buildStartGameBlock(myProvider)
                : _buildScoreContainer(myProvider),
            SizedBox(height: 2),
            _buildImageItemsBlock(myProvider),
          ],
        ),
      ),
    );

    return SizedBox.expand(
      child: Container(
        child: FittedBox(
          fit: BoxFit.contain,
          alignment: Alignment.center,
          child: SizedBox(
            height: (MediaQuery.of(context).size.height),
            width: (MediaQuery.of(context).size.width),
            child: pageContent,
          ),
        ),
      ),
    );
  }
}
