import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:wordguessing/models/word.dart';
import 'package:wordguessing/utils/random_pick_words.dart';

import '../provider/data.dart';

class GamePickWordPage extends StatelessWidget {
  final int _countWords = 4;
  final int _countImages = 4;

  Future<void> startGame(Data myProvider, String lang) async {
    await resetGame(myProvider);
    myProvider.updateLang = lang;
    await nextWord(myProvider);
  }

  Future<void> resetGame(Data myProvider) async {
    myProvider.updateLang = '';
    myProvider.updateQuestionsCount = 0;
    myProvider.updateGoodAnswers = 0;
    myProvider.updateWrongAnswers = 0;
    myProvider.updateWord = null;
    myProvider.updateImages = [];
  }

  Future<void> nextWord(Data myProvider) async {
    await pickData(myProvider);
    myProvider.updateQuestionsCount = myProvider.questionsCount + 1;
  }

  Future<void> pickData(Data myProvider) async {
    List<Word> words = [];
    List<Word> images = [];
    RandomPickWords randomPickWords;
    Word word;

    int attempts = 0;
    do {
      randomPickWords = RandomPickWords();
      await randomPickWords.init(myProvider.lang, _countWords);

      words = randomPickWords.words;

      word = words.take(1).toList()[0];
      images = [word];

      attempts++;

      if ((words.length >= _countWords) &&
          // (images.length >= _countImages) &&
          !myProvider.isRecentlyPicked(word.key)) {
        myProvider.updateWord = word;
        myProvider.updateOtherWords = words.skip(1).toList();
        myProvider.updateImages = images;
      }
    } while (myProvider.word != word && attempts < 10);
  }

  Future<void> checkWord(Data myProvider, word) async {
    if (myProvider.word?.key == word.key) {
      myProvider.updateGoodAnswers = myProvider.goodAnswers + 1;
      nextWord(myProvider);
    } else {
      myProvider.updateWrongAnswers = myProvider.wrongAnswers + 1;
    }
  }

  Container _buildScoreItemContainer(String text, Color blockColor) {
    Color backgroundColor = blockColor;

    // Darken block color to get border color
    double amount = 0.2;
    final hsl = HSLColor.fromColor(blockColor);
    final hslDark = hsl.withLightness((hsl.lightness - amount).clamp(0.0, 1.0));
    Color borderColor = hslDark.toColor();

    return Container(
      margin: EdgeInsets.symmetric(horizontal: 15),
      padding: EdgeInsets.all(5),
      decoration: BoxDecoration(
        color: backgroundColor,
        borderRadius: BorderRadius.circular(4),
        border: Border.all(
          color: borderColor,
          width: 4,
        ),
      ),
      child: Text(
        text,
        style: TextStyle(
          fontSize: 25,
          fontWeight: FontWeight.w600,
          color: Colors.black,
        ),
      ),
    );
  }

  String getGoodAnswersString(Data myProvider) {
    int count = myProvider.questionsCount - 1;
    return '👍 ' + myProvider.goodAnswers.toString() + '/' + count.toString();
  }

  String getWrongAnswersString(Data myProvider) {
    int count = myProvider.questionsCount - 1;
    return '🚩 ' + myProvider.wrongAnswers.toString() + '/' + count.toString();
  }

  Container _buildScoreContainer(Data myProvider) {
    return Container(
      padding: EdgeInsets.all(5),
      child: Row(
        mainAxisSize: MainAxisSize.min,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          _buildScoreItemContainer(
            getGoodAnswersString(myProvider),
            Colors.green,
          ),
          SizedBox(width: 20),
          _buildScoreItemContainer(
            getWrongAnswersString(myProvider),
            Colors.orange,
          ),
        ],
      ),
    );
  }

  Container _buildImageContainer(String image) {
    double imageSize = 130;

    String imageAsset = 'assets/placeholder.png';
    if (image != '') {
      imageAsset = 'assets/images/' + image;
    }

    return Container(
      margin: EdgeInsets.all(2),
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(8),
        border: Border.all(
          color: Colors.blue.shade200,
          width: 8,
        ),
      ),
      child: Image(
          image: AssetImage(imageAsset),
          width: imageSize,
          height: imageSize,
          fit: BoxFit.fill),
    );
  }

  Column _buildImageItemsBlock(Word? currentWord) {
    if ((currentWord?.images.length ?? 0) < _countImages) {
      return Column();
    }

    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        Row(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            _buildImageContainer(currentWord?.images[0] ?? ''),
            _buildImageContainer(currentWord?.images[1] ?? ''),
          ],
        ),
        Row(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            _buildImageContainer(currentWord?.images[2] ?? ''),
            _buildImageContainer(currentWord?.images[3] ?? ''),
          ],
        )
      ],
    );
  }

  Container _buildTextContainer(Data myProvider, Word word) {
    return Container(
      margin: EdgeInsets.all(2),
      decoration: BoxDecoration(
        color: Colors.blue[800],
        borderRadius: BorderRadius.circular(6),
        border: Border.all(
          color: Colors.white,
          width: 6,
        ),
      ),
      child: TextButton(
        style: TextButton.styleFrom(
          padding: EdgeInsets.all(15),
        ),
        child: Text(
          word.text,
          style: TextStyle(
            fontSize: 20,
            fontWeight: FontWeight.w600,
            color: Colors.white,
          ),
        ),
        onPressed: () {
          checkWord(myProvider, word);
        },
      ),
    );
  }

  Column _buildTextItemsBlock(Data myProvider) {
    Word? word = myProvider.word;
    List otherWords = myProvider.otherWords;

    if ((word == null) || (otherWords.length != (_countWords - 1))) {
      return Column();
    }

    List words = [
      word,
      otherWords.length > 0 ? otherWords[0] : null,
      otherWords.length > 1 ? otherWords[1] : null,
      otherWords.length > 2 ? otherWords[2] : null,
    ];

    words.sort((a, b) => a.key.compareTo(b.key));

    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        Row(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            _buildTextContainer(myProvider, words[0]),
            SizedBox(width: 10),
            _buildTextContainer(myProvider, words[1]),
          ],
        ),
        SizedBox(height: 5),
        Row(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            _buildTextContainer(myProvider, words[2]),
            SizedBox(width: 10),
            _buildTextContainer(myProvider, words[3]),
          ],
        )
      ],
    );
  }

  Column _buildStartGameBlock(Data myProvider) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        Container(
          child: ElevatedButton(
            style: ElevatedButton.styleFrom(
              foregroundColor: Colors.teal,
              backgroundColor: Colors.teal,
              padding: EdgeInsets.all(35),
            ),
            child: Text(
              "🇫🇷",
              style: TextStyle(
                fontSize: 60,
                color: Colors.black,
              ),
            ),
            onPressed: () => startGame(myProvider, 'fr'),
          ),
        ),
        SizedBox(height: 15),
        Container(
          child: ElevatedButton(
            style: ElevatedButton.styleFrom(
              foregroundColor: Colors.teal,
              backgroundColor: Colors.teal,
              padding: EdgeInsets.all(35),
            ),
            child: Text(
              "🇬🇧",
              style: TextStyle(
                fontSize: 60,
                color: Colors.black,
              ),
            ),
            onPressed: () => startGame(myProvider, 'en'),
          ),
        )
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    Data myProvider = Provider.of<Data>(context);

    Scaffold pageContent = Scaffold(
      appBar: AppBar(
        elevation: 0,
        actions: <Widget>[
          IconButton(
            icon: const Icon(Icons.loop),
            onPressed: () => resetGame(myProvider),
          ),
        ],
      ),
      backgroundColor: Colors.blue,
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
          crossAxisAlignment: CrossAxisAlignment.center,
          mainAxisSize: MainAxisSize.max,
          children: <Widget>[
            (myProvider.images.length != 0)
                ? _buildImageItemsBlock(myProvider.images[0])
                : SizedBox.shrink(),
            SizedBox(height: 2),
            ((myProvider.word == null) || (myProvider.word?.key == ''))
                ? _buildStartGameBlock(myProvider)
                : _buildScoreContainer(myProvider),
            SizedBox(height: 2),
            _buildTextItemsBlock(myProvider),
          ],
        ),
      ),
    );

    return SizedBox.expand(
      child: Container(
        child: FittedBox(
          fit: BoxFit.contain,
          alignment: Alignment.center,
          child: SizedBox(
            height: (MediaQuery.of(context).size.height),
            width: (MediaQuery.of(context).size.width),
            child: pageContent,
          ),
        ),
      ),
    );
  }
}
